﻿-- don't load if class is wrong
local _, class = UnitClass("player")
if class ~= "WARRIOR" then return end

if not clcInfo then end
local GetTime = GetTime
local debug = clcInfo.debug

local version = 1

local modName = "____arms_warrior"
local mod = clcInfo:RegisterClassModule(modName)
local emod = clcInfo.env
local db
local target_health = 100

local db_default = {
	char_var = false,
}


local tdb_default = {
	version = version,
	
	prio = "rend cs ms op exe slam",
}
-- @defines
--------------------------------------------------------------------------------
local gcdId 				= 85256 -- tv for gcd
-- list of spellId

local rendId				= 772	-- rend
local csId                  = 86346 -- colossus smash
local msId                  = 12294 -- mortal strike
local opId                  = 7384  -- overpower
local exeId                 = 5308  -- execute
local slamId                = 1464  -- slam
local hsId					= 78	-- heroic strike
local irId					= 1134	-- inner rage

-- devName to pass as argument for melee range checks
local slamName			= GetSpellInfo(slamId)
-- buffs

--Spells for localization
local spells ={
		["Rend"]=rendId,
        ["Colossus Smash"]=csId,
}

--localize spells
local name
for key, value in pairs(spells) do
	local name = GetSpellInfo(value)
	if not name then name = "Unknown Spell" end
	spells[key] = { id = value, name = name }
end

-- status vars
local s1, s2
-- current time, old time, remaining gcd, current gcd duration
local s_ctime, s_otime, s_gcd, s_gcdDur

local function CalculateCurrentTime()
    return s_ctime + s_gcd + s_gcdDur
end

local function CalculateTargetHealthPct()
    return ((UnitHealth("target") / UnitHealthMax("target")) * 100)
end

function mod:OnTemplatesUpdate()
	db = clcInfo:RegisterClassModuleTDB(modName, tdb_default)
	if db then
		if not db.version or db.version < version then
        -- fix stuff
			clcInfo.AdaptConfigAndClean(modName, db, default)
			db.version = version
		end
		
		mod:UpdateQueue()
	end
end


function mod.OnInitialize()
	db = clcInfo:RegisterClassModuleDB(modName, db_default)
end

-- the queue
local q = {}

local function GetCooldown(id)
	local start, duration = GetSpellCooldown(id)
	local cd = start + duration - s_ctime - s_gcd
	if cd < 0 then return 0 end
	return cd
end

-- actions ---------------------------------------------------------------------
local actions = {
	rend = {
		id = rendId,
		GetCD = function()
            local visible = emod.IconAura("HARMFUL|PLAYER", "target", spells["Rend"].name)
            if not visible and target_health > 20 then
			    if s1 == rendId then
				    return (1.5 - s_gcd)
			    else
				    if GetCooldown(rendId) < .2 then
					    return 0
				    end
				    return GetCooldown(rendId)
                end
			end
            return 100
		end,
		UpdateStatus = function()
            target_health = CalculateTargetHealthPct()
			s_ctime = CalculateCurrentTime()
		end,
		info = "Rend",
	},
	cs = {
		id = csId,
		GetCD = function()
            local _, _, _, _, _, _, expires, caster, _ = UnitDebuff("target", "Colossus Smash")
            local time_left = 0
            if expires ~= nil and caster == "player" then
                time_left = expires - GetTime()
            end
            if s1 ~= csId then
                local cooldown = GetCooldown(csId)
                if time_left > cooldown then
                    cooldown = time_left
                end
				return cooldown
			end
			return 100
		end,
		UpdateStatus = function()
            target_health = CalculateTargetHealthPct()
			s_ctime = CalculateCurrentTime()
		end,
		info = "Colossus Smash",
	},
    ms = {
		id = msId,
		GetCD = function()
			if s1 ~= msId then
				return GetCooldown(msId)
			end
			return 100
		end,
		UpdateStatus = function()
            target_health = CalculateTargetHealthPct()
			s_ctime = CalculateCurrentTime()
		end,
		info = "Mortal Strike",
	},
	op = {
		id = opId,
		GetCD = function()
            local taste_for_blood, _, _, _, _, _, _ = emod.IconAura("HELPFUL|PLAYER", "player", "Taste for Blood")
			if s1 ~= opId and taste_for_blood == true and target_health > 20 then
                return GetCooldown(opId)
			end
			return 100
		end,
		UpdateStatus = function()
            target_health = CalculateTargetHealthPct()
			s_ctime = CalculateCurrentTime()
		end,
		info = "Overpower",
	},
	exe = {
		id = exeId,
		GetCD = function()
			if IsUsableSpell(exeId) then
				return GetCooldown(exeId)
			end
			return 100
		end,
		UpdateStatus = function()
            target_health = CalculateTargetHealthPct()
			s_ctime = CalculateCurrentTime()
		end,
		info = "Execute",
	},
    slam = {
		id = slamId,
		GetCD = function()
			return GetCooldown(slamId)
		end,
		UpdateStatus = function()
            target_health = CalculateTargetHealthPct()
			s_ctime = CalculateCurrentTime()
		end,
		info = "Slam",
	},
}
mod.actions = actions
--------------------------------------------------------------------------------

function mod:UpdateQueue()
	q = {}
	for v in string.gmatch(db.prio, "[^ ]+") do
        if actions[v] then
            table.insert(q, v)
        else
            print("clcInfo", modName, "invalid action:", v)
        end
    end
	db.prio = table.concat(q, " ")
end

-- reads all the interesting data
local function GetStatus()
	-- current time
	s_ctime = GetTime()
	
	-- gcd value
	local start, duration = GetSpellCooldown(gcdId)
	s_gcd = start + duration - s_ctime
	s_gcdDur = duration
	if s_gcd < 0 then s_gcd = 0 end
end

local function GetNextAction()
	local n = #q
	
	-- parse once, get cooldowns, return first 0
	for i = 1, n do
		local action = actions[q[i]]
		local cd = action.GetCD()
		if debug.enabled then
			debug:AddBoth(q[i], cd)
		end
		if cd == 0 then
			return action.id, q[i]
		end
		action.cd = cd
	end
	
	-- parse again, return min cooldown
	local minQ = 1
	local minCd = actions[q[1]].cd
	for i = 2, n do
		local action = actions[q[i]]
		if minCd > action.cd then
			minCd = action.cd
			minQ = i
		end
	end
	return actions[q[minQ]].id, q[minQ]
end

function ArmsRotation()
	-- get the current db
	s1 = nil
	GetStatus()
	local action
	s1, action = GetNextAction()
	s_otime = s_ctime -- save it so we adjust buffs for next
	actions[action].UpdateStatus()
	
	-- adjust buffs
	s_otime = s_ctime - s_otime
	s2, action = GetNextAction()
	
	return s1, s2
end


-- plug in
--------------------------------------------------------------------------------
local nextSkill
function emod.IconArmsCurrent(...)
	ArmsRotation(...)
	if nextSkill then nextSkill:DoUpdate() end
	return emod.IconSpell(s1, db.rangePerSkill or devName)
end
local function NextExec()
	return emod.IconSpell(s2, db.rangePerSkill or devName)
end
local function ExecCleanupNext()
	nextSkill = nil
end
function emod.IconArmsNext(...)
	nextSkill = emod.___e
	nextSkill:SetScript("OnUpdate", nil)
	nextSkill.exec = NextExec
	nextSkill.ExecCleanup = ExecCleanupNext
end

-- giving queue as command line
local function CmdArms(args)
	db.prio = table.concat(args, " ")
	mod:UpdateQueue()
	clcInfo:UpdateOptions()
end
clcInfo.cmdList["armsprio"] = CmdArms